@@ -218,6 +218,23 @@ get_backing_file (const char *virtual_path)
 	return NULL;
 }
 
+static gboolean
+ask_disable_joliet (GtkWindow *parent)
+{
+	GtkWidget *dialog;
+	int res;
+
+	dialog = gtk_message_dialog_new (parent,
+					 GTK_DIALOG_DESTROY_WITH_PARENT,
+					 GTK_MESSAGE_QUESTION,
+					 GTK_BUTTONS_OK_CANCEL,
+					 _("Some files don't have a suitable name for a Windows-compatible CD.\nDo you want to continue with Windows compatibility disabled?"));
+	gtk_window_set_title (GTK_WINDOW (dialog), _("Windows compatibility"));
+	res = gtk_dialog_run (GTK_DIALOG (dialog));
+	gtk_widget_destroy (dialog);
+	return (res == GTK_RESPONSE_OK);
+}
+
 struct mkisofs_state {
 	FILE *graft_file;
 	char *emptydir;
@@ -473,7 +490,8 @@ cancel_mkisofs (gpointer data)
  * Create an ISO image in filename from the data files in burn:///
  */
 int
-make_iso (const char *filename, const char *label, gboolean warn_low_space)
+make_iso (const char *filename, const char *label,
+		gboolean warn_low_space, gboolean use_joliet)
 {
 	GnomeVFSURI *uri;
 	char *filelist = NULL;
@@ -487,7 +505,7 @@ make_iso (const char *filename, const char *label, gboolean warn_low_space)
 	struct mkisofs_output mkisofs_output;
 	GIOChannel *channel;
 	guint stdout_tag, stderr_tag;
-	char *stdout_data;
+	char *stdout_data, *stderr_data;
 	char *dirname;
 	int exit_status, res;
 	unsigned long iso_size;
@@ -546,24 +564,28 @@ make_iso (const char *filename, const char *label, gboolean warn_low_space)
 		mkisofs_output.result = RESULT_ERROR;
 		goto cleanup;
 	}
-	
+
+retry:
 	i = 0;
 	argv[i++] = "mkisofs";
 	argv[i++] = "-r";
-	argv[i++] = "-J";
+	if (use_joliet) {
+		argv[i++] = "-J";
+	}
+	argv[i++] = "-q";
 	argv[i++] = "-graft-points";
 	argv[i++] = "-path-list";
 	argv[i++] = filelist;
 	argv[i++] = "-print-size";
 	argv[i++] = NULL;
-	
+
 	if (!g_spawn_sync (NULL,
 			   (char **)argv,
 			   NULL,
-			   G_SPAWN_SEARCH_PATH | G_SPAWN_STDERR_TO_DEV_NULL,
+			   G_SPAWN_SEARCH_PATH,
 			   NULL, NULL,
 			   &stdout_data,
-			   NULL,
+			   &stderr_data,
 			   &exit_status,
 			   NULL)) {
 		g_warning ("mkisofs command failed: %s\n", error->message);
@@ -572,6 +594,22 @@ make_iso (const char *filename, const char *label, gboolean warn_low_space)
 		mkisofs_output.result = RESULT_ERROR;
 		goto cleanup;
 	}
+
+	if (exit_status != 0 && use_joliet) {
+		if (strstr (stderr_data, "Joliet tree sort failed.") != NULL) {
+			g_free (stdout_data);
+			g_free (stderr_data);
+			if (ask_disable_joliet (cd_progress_get_window ())) {
+				use_joliet = FALSE;
+				goto retry;
+			} else {
+				mkisofs_output.result = RESULT_ERROR;
+				goto cleanup;
+			}
+		}
+	}
+
+	g_free (stderr_data);
 	iso_size = atol (stdout_data); /* blocks of 2048 bytes */
 	g_free (stdout_data);
 
@@ -610,7 +648,9 @@ make_iso (const char *filename, const char *label, gboolean warn_low_space)
 	i = 0;
 	argv[i++] = "mkisofs";
 	argv[i++] = "-r";
-	argv[i++] = "-J";
+	if (use_joliet) {
+		argv[i++] = "-J";
+	}
 	argv[i++] = "-graft-points";
 	argv[i++] = "-path-list";
 	argv[i++] = filelist;
@@ -715,8 +755,8 @@ build_list_of_audio_tracks (void)
 			backing = get_backing_file (info->name);
 			track = g_new0 (Track, 1);
 			track->type = TRACK_TYPE_AUDIO;
-			track->audio.filename = backing;
-			track->audio.cdtext = NULL;
+			track->contents.audio.filename = backing;
+			track->contents.audio.cdtext = NULL;
 			list = g_list_prepend (list, track);
 		}
 	}
